//
//  TrendingHashtagsViewController.swift
//  Tusker
//
//  Created by Shadowfacts on 2/6/21.
//  Copyright © 2021 Shadowfacts. All rights reserved.
//

import UIKit
import Pachyderm

class TrendingHashtagsViewController: EnhancedTableViewController {

    weak var mastodonController: MastodonController!
    
    private var dataSource: UITableViewDiffableDataSource<Section, Item>!
    
    init(mastodonController: MastodonController) {
        self.mastodonController = mastodonController
        
        super.init(style: .grouped)
    }
    
    required init?(coder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
    override func viewDidLoad() {
        super.viewDidLoad()

        title = NSLocalizedString("Trending Hashtags", comment: "trending hashtags screen title")
        
        tableView.register(UINib(nibName: "TrendingHashtagTableViewCell", bundle: .main), forCellReuseIdentifier: "trendingTagCell")
        tableView.rowHeight = 60 // 44 for content + 2 * 8 spacing
        
        // todo: enable drag
        
        dataSource = UITableViewDiffableDataSource<Section, Item>(tableView: tableView) { (tableView, indexPath, item) in
            switch item {
            case let .tag(hashtag):
                let cell = tableView.dequeueReusableCell(withIdentifier: "trendingTagCell", for: indexPath) as! TrendingHashtagTableViewCell
                cell.updateUI(hashtag: hashtag)
                return cell
            }
        }
    }
    
    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)
        
        let request = Client.getTrends(limit: 10)
        mastodonController.run(request) { (response) in
            var snapshot = NSDiffableDataSourceSnapshot<Section, Item>()
            
            guard case let .success(hashtags, _) = response,
                  hashtags.count > 0 else {
                self.dataSource.apply(snapshot)
                return
            }
            
            snapshot.appendSections([.trendingTags])
            snapshot.appendItems(hashtags.map { .tag($0) })
            self.dataSource.apply(snapshot)
        }
    }

    // MARK: - Table View Delegate
    
    override func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        guard let item = dataSource.itemIdentifier(for: indexPath),
              case let .tag(hashtag) = item else {
            return
        }
        
        show(HashtagTimelineViewController(for: hashtag, mastodonController: mastodonController), sender: nil)
    }
    
}

extension TrendingHashtagsViewController {
    enum Section {
        case trendingTags
    }
    enum Item: Hashable {
        case tag(Hashtag)
    }
}
